@@ -79,7 +79,7 @@ static unsigned int leaf_data_end(const struct extent_buffer *leaf)
  * have to adjust any offsets to account for the header in the leaf.  This
  * handles that math to simplify the callers.
  */
-static inline void memmove_leaf_data(const struct extent_buffer *leaf,
+static inline void memmove_leaf_data(struct extent_buffer *leaf,
 				     unsigned long dst_offset,
 				     unsigned long src_offset,
 				     unsigned long len)
@@ -102,7 +102,7 @@ static inline void memmove_leaf_data(const struct extent_buffer *leaf,
  * have to adjust any offsets to account for the header in the leaf.  This
  * handles that math to simplify the callers.
  */
-static inline void copy_leaf_data(const struct extent_buffer *dst,
+static inline void copy_leaf_data(struct extent_buffer *dst,
 				  const struct extent_buffer *src,
 				  unsigned long dst_offset,
 				  unsigned long src_offset, unsigned long len)
@@ -122,7 +122,7 @@ static inline void copy_leaf_data(const struct extent_buffer *dst,
  * Wrapper around memmove_extent_buffer() that does the math to get the
  * appropriate offsets into the leaf from the item numbers.
  */
-static inline void memmove_leaf_items(const struct extent_buffer *leaf,
+static inline void memmove_leaf_items(struct extent_buffer *leaf,
 				      int dst_item, int src_item, int nr_items)
 {
 	memmove_extent_buffer(leaf, btrfs_item_nr_offset(leaf, dst_item),
@@ -142,7 +142,7 @@ static inline void memmove_leaf_items(const struct extent_buffer *leaf,
  * Wrapper around copy_extent_buffer() that does the math to get the
  * appropriate offsets into the leaf from the item numbers.
  */
-static inline void copy_leaf_items(const struct extent_buffer *dst,
+static inline void copy_leaf_items(struct extent_buffer *dst,
 				   const struct extent_buffer *src,
 				   int dst_item, int src_item, int nr_items)
 {
@@ -232,9 +232,9 @@ noinline void btrfs_release_path(struct btrfs_path *p)
  * cause could be a bug, eg. due to ENOSPC, and not for common errors that are
  * caused by external factors.
  */
-bool __cold abort_should_print_stack(int errno)
+bool __cold abort_should_print_stack(int error)
 {
-	switch (errno) {
+	switch (error) {
 	case -EIO:
 	case -EROFS:
 	case -ENOMEM:
@@ -1250,8 +1250,8 @@ int btrfs_find_item(struct btrfs_root *fs_root, struct btrfs_path *found_path,
 	struct extent_buffer *eb;
 	struct btrfs_path *path;
 
-	key.type = key_type;
 	key.objectid = iobjectid;
+	key.type = key_type;
 	key.offset = ioff;
 
 	if (found_path == NULL) {
